@@ -20,7 +20,7 @@
     after TDR was copied into TSR, which is approximatively when the next bit should
     be transferred (256 cycles) (fix the program 'Notator')
 */
-const char Midi_fileid[] = "Hatari midi.c : " __DATE__ " " __TIME__;
+const char Midi_fileid[] = "Hatari midi.c";
 
 #include <SDL_types.h>
 
@@ -166,13 +166,13 @@ static void	MIDI_UpdateIRQ ( void )
 		if ( irq_bit_new )
 		{
 			/* Request interrupt by setting GPIP to low/0 */
-			MFP_GPIP_Set_Line_Input ( MFP_GPIP_LINE_ACIA , MFP_GPIP_STATE_LOW );
+			MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE_ACIA , MFP_GPIP_STATE_LOW );
 			MidiStatusRegister |= ACIA_SR_INTERRUPT_REQUEST;
 		}
 		else
 		{
 			/* Clear interrupt request by setting GPIP to high/1 */
-			MFP_GPIP_Set_Line_Input ( MFP_GPIP_LINE_ACIA , MFP_GPIP_STATE_HIGH );
+			MFP_GPIP_Set_Line_Input ( pMFP_Main , MFP_GPIP_LINE_ACIA , MFP_GPIP_STATE_HIGH );
 			MidiStatusRegister &= ~ACIA_SR_INTERRUPT_REQUEST;
 		}
 	}
@@ -614,10 +614,13 @@ static bool Midi_Host_WriteByte(Uint8 byte)
 		PmEvent* midiEvent = Midi_BuildEvent(byte);
 		if (midiEvent)
 		{
+			const char *msg;
 			PmError error = Pm_Write(midiOut, midiEvent, 1);
-			if (error == pmNoError)
+			if (error == pmNoError || error == pmGotData)
 				return true;
-			LOG_TRACE(TRACE_MIDI, "MIDI: PortMidi write error %d\n", error);
+			msg = Pm_GetErrorText(error);
+			Log_Printf(LOG_WARN, "MIDI: PortMidi write error %d: '%s'\n",
+				  error, msg);
 			return false;
 		}
 		return true;
