@@ -1,6 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 #
-# Copyright (C) 2012-2019 by Eero Tamminen <oak at helsinkinet fi>
+# Copyright (C) 2012-2020 by Eero Tamminen <oak at helsinkinet fi>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -106,12 +106,12 @@ class TOS:
         # older TOS versions don't support autostarting
         # programs from GEMDOS HD dir with *.INF files
         f.seek(0x2C, 0)
-        etos = (f.read(4) == "ETOS")
+        etos = (f.read(4) == b"ETOS")
         return (version, etos)
 
 
     def _add_info(self):
-        "add TOS version specific info of supported machines etc"
+        "add TOS version specific info of supported machines etc, return timeouts & supported machines"
         name, size, version = self.name, self.size, self.version
 
         if self.etos:
@@ -128,27 +128,30 @@ class TOS:
         # https://en.wikipedia.org/wiki/Atari_TOS
         elif version <= 0x100:
             # boots up really slow with 4MB
-            info = (0, 16, ("st",))
+            info = (0, 20, ("st",))
         elif version <= 0x104:
-            info = (0, 6, ("st", "megast"))
-        elif version < 0x200:
-            info = (0, 6, ("ste",))
-        elif version < 0x300:
+            info = (0, 12, ("st", "megast"))
+        elif version <= 0x162:
+            info = (0, 20, ("ste",))
+        elif version < 0x206:
             # TOS v2.x are slower with VDI mode than others
-            info = (2, 8, ("st", "megast", "ste", "megaste"))
-        elif version < 0x400:
-            # memcheck comes up fast, but boot takes time
-            info = (2, 8, ("tt",))
-        elif version < 0x500:
-            # memcheck takes long to come up with 14MB
-            info = (3, 8, ("falcon",))
+            info = (3, 14, ("ste", "megaste"))
+        elif version == 0x206:
+            # ST support added to TOS 2.x only after 2.05
+            info = (3, 14, ("st", "megast", "ste", "megaste"))
+        elif version <= 0x306:
+            # MMU slowdown is taken care of in prepare_test()
+            info = (3, 16, ("tt",))
+        elif version <= 0x404:
+            # no-IDE scan slowdown is taken care of in prepare_test()
+            info = (3, 28, ("falcon",))
         else:
-            raise AssertionError("'%s' TOS version 0x%x isn't valid" % (name, version))
+            raise AssertionError("Unknown '%s' TOS version 0x%x" % (name, version))
 
         if self.etos:
-            print("%s is EmuTOS v%x %dkB" % (name, version, size))
+            print("%s is %dkB EmuTOS corresponding to TOS v%x (wait startup: %ds, rest: %ds)" % (name, size, version, info[0], info[1]))
         else:
-            print("%s is normal TOS v%x" % (name, version))
+            print("%s is normal TOS v%x (wait memcheck: %ds, rest: %ds)" % (name, version, info[0], info[1])
         # 0: whether / how long to wait to dismiss memory test
         # 1: how long to wait until concluding test failed
         # 2: list of machines supported by this TOS version
@@ -172,6 +175,7 @@ class TOS:
         # TOS isn't support.
         #
         # (And even with a driver, only TOS 4.x supports IDE.)
+        print("NOTE: '%s' hard disk tests are supported only for EmuTOS" % hdinterface)
         return False
 
     def supports_monitor(self, monitortype, machine):
@@ -195,9 +199,13 @@ class TOS:
                 return False
         return True
 
-    def supports_32bit_addressing(self):
+    def supports_32bit_addressing(self, disk):
         "whether TOS version supports 32-bit addressing"
-        if self.etos or self.version >= 0x300:
+        if self.etos or (self.version >= 0x300 and self.version < 0x400):
+            return True
+        # Hatari patches TOS v4 for 32-bit support, but TOS v4 floppy access doesn't
+        # work with TT-RAM as there's no _FRB cookie pointing to 64K ST-RAM DMA buffer
+        if self.version >= 0x400 and disk != 'floppy':
             return True
         return False
 
@@ -217,6 +225,7 @@ class Config:
 
     # defaults
     fast = False
+    opts = []
     bools = []
     disks = ("floppy", "gemdos", "scsi")
     graphics = ("mono", "rgb", "vga", "vdi1", "vdi4")
@@ -225,15 +234,21 @@ class Config:
     ttrams = (0, 32)
 
     def __init__(self, argv):
-        longopts = ["bool=", "disks=", "fast", "graphics=", "help", "machines=", "memsizes=", "ttrams="]
+        longopts = ["bool=", "disks=", "fast", "graphics=", "help", "machines=", "memsizes=", "opts=", "ttrams="]
         try:
-            opts, paths = getopt.gnu_getopt(argv[1:], "b:d:fg:hm:s:t:", longopts)
+            opts, paths = getopt.gnu_getopt(argv[1:], "b:d:fg:hm:s:o:t:", longopts)
         except getopt.GetoptError as error:
             self.usage(error)
         self.handle_options(opts)
         self.images = self.check_images(paths)
-        configs = (self.disks, self.graphics, self.machines, self.memsizes, self.ttrams, self.bools)
-        print("Test configuration:\n\t%s %s %s RAM=%s TTRAM=%s bool=%s\n" % configs)
+        print("\nTest configurations:")
+        print("- machines = %s" % self.machines)
+        print("- graphics = %s" % self.graphics)
+        print("- disks = %s" % self.disks)
+        print("- RAM = %s" % self.memsizes)
+        print("- TTRAM = %s" % self.ttrams)
+        print("- bools = %s" % self.bools)
+        print("- fixed = '%s'\n" % ' '.join(self.opts))
 
 
     def check_images(self, paths):
@@ -259,7 +274,9 @@ class Config:
             if opt in ("-f", "--fast"):
                 self.fast = True
             elif opt in ("-b", "--bool"):
-                self.bools += args
+                self.bools = args
+            elif opt in ("-o", "--opts"):
+                self.opts = arg.split()
             elif opt in ("-d", "--disks"):
                 unknown, self.disks = validate(args, self.all_disks)
             elif opt in ("-g", "--graphics"):
@@ -288,6 +305,10 @@ class Config:
     def usage(self, msg=None):
         "output program usage information"
         name = os.path.basename(sys.argv[0])
+        # option value lists in directly CLI copy-pastable format
+        disks, graphics, machines = [','.join(x) for x in
+            (self.all_disks, self.all_graphics, self.all_machines)]
+        memsizes = ','.join([str(x) for x in self.all_memsizes])
         print(__doc__)
         print("""
 Usage: %s [options] <TOS image files>
@@ -296,14 +317,16 @@ Options:
 \t-h, --help\tthis help
 \t-f, --fast\tspeed up boot with less accurate emulation:
 \t\t\t"--fast-forward yes --fast-boot yes --fastfdc yes --timer-d yes"
-\t-d, --disks\t%s
-\t-g, --graphics\t%s
-\t-m, --machines\t%s
-\t-s, --memsizes\t%s
-\t-t, --ttrams\t%s
+\t-d, --disks\t(%s)
+\t-g, --graphics\t(%s)
+\t-m, --machines\t(%s)
+\t-s, --memsizes\t(%s)
+\t-t, --ttrams\t(0-512, in 4MB steps)
 \t-b, --bool\t(extra boolean Hatari options to test)
+\t-o, --opts\t(hatari options to pass as-is)
 
-Multiple values for an option need to be comma separated. If some
+Multiple values for an option need to be comma separated. If option
+is given multiple times, last given value(s) are used. If some
 option isn't given, default list of values will be used for that.
 
 For example:
@@ -313,8 +336,9 @@ For example:
 \t--memsizes 0,4,14 \\
 \t--ttrams 0,32 \\
 \t--graphics mono,rgb \\
-\t--bool --compatible,--mmu
-""" % (name, self.all_disks, self.all_graphics, self.all_machines, self.all_memsizes, self.ttrams, name))
+\t--bool --compatible,--driver-b \\
+\t--opts "--mmu on"
+""" % (name, disks, graphics, machines, memsizes, name))
         if msg:
             print("ERROR: %s\n" % msg)
         sys.exit(1)
@@ -327,9 +351,9 @@ For example:
         if disktype == "gemdos":
             return tos.supports_gemdos_hd()
 
-        if machine in ("st", "megast", "ste"):
+        if machine in ("st", "megast", "ste", "megaste"):
             hdinterface = ("acsi",)
-        elif machine in ("megaste", "tt"):
+        elif machine == "tt":
             hdinterface = ("acsi", "scsi")
         elif machine == "falcon":
             hdinterface = ("ide", "scsi")
@@ -367,7 +391,7 @@ For example:
             return True
         return False
 
-    def valid_ttram(self, machine, tos, ttram, winuae):
+    def valid_ttram(self, machine, tos, ttram, disk, winuae):
         "return whether given TT-RAM size is valid for given machine"
         if machine in ("st", "megast", "ste", "megaste"):
             if ttram == 0:
@@ -380,7 +404,7 @@ For example:
                 return False
             if ttram < 0 or ttram > 512:
                 return False
-            return tos.supports_32bit_addressing()
+            return tos.supports_32bit_addressing(disk)
         else:
             raise AssertionError("unknown machine %s" % machine)
         return False
@@ -393,8 +417,13 @@ For example:
         #
         # Below ones should be potentially relevant ones to test
         # (EmuTOS supports NatFeats so it can have impact too)
-        generic_opts = ("--compatible", "--timer-d", "--fastfdc", "--fast-boot", "--natfeats")
-        winuae_opts = ("--cpu-exact", "--mmu", "--addr24", "--fpu-softfloat")
+        generic_opts = (
+            "--compatible", "--timer-d", "--fast-boot",
+            "--natfeats", "--fastfdc", "--drive-b"
+        )
+        winuae_opts = (
+            "--cpu-exact", "--mmu", "--addr24", "--fpu-softfloat"
+        )
         for option in self.bools:
             if option not in generic_opts:
                 if option not in winuae_opts:
@@ -413,23 +442,36 @@ For example:
         return True
 
 # -----------------------------------------------
-def verify_file_match(srcfile, dstfile):
-    "return error string if sizes of given files don't match"
+def verify_file_match(srcfile, dstfile, identity):
+    "return error string if sizes of given files don't match, and rename dstfile to identity"
     if not os.path.exists(dstfile):
         return "file '%s' missing" % dstfile
     dstsize = os.stat(dstfile).st_size
     srcsize = os.stat(srcfile).st_size
     if dstsize != srcsize:
+        os.rename(dstfile, "%s.%s" % (dstfile, identity))
         return "file '%s' size %d doesn't match file '%s' size %d" % (srcfile, srcsize, dstfile, dstsize)
+    return None
 
-def verify_file_empty(filename):
-    "return error string if given file isn't empty"
+def verify_file_empty(filename, identity):
+    "return error string if given file isn't empty, and rename file to identity"
     if not os.path.exists(filename):
         return "file '%s' missing" % filename
     size = os.stat(filename).st_size
     if size != 0:
+        os.rename(filename, "%s.%s" % (filename, identity))
         return "file '%s' isn't empty (%d bytes)" % (filename, size)
 
+def exit_if_missing(names):
+    "exit if given (test input) file is missing"
+    for name in names:
+        if not os.path.exists(name):
+            error_exit("test file '%s' missing")
+
+
+# how long to wait for invoked Hatari to open FIFO (= MIDI output file)
+FIFO_WAIT = 5
+
 class Tester:
     "test driver class"
     output = "output" + os.path.sep
@@ -443,11 +485,15 @@ class Tester:
     printout  = output + "printer-out"
     serialout = output + "serial-out"
     fifofile  = output + "midi-out"
-    bootauto  = "bootauto.st.gz" # TOS old not to support GEMDOS HD either
+    # oldest TOS versions don't support GEMDOS HD or auto-starting,
+    # they need to use floppy and boot test program from AUTO/
+    bootauto  = "bootauto.st.gz"
     bootdesk  = "bootdesk.st.gz"
     floppyprg = "A:\MINIMAL.PRG"
+    # with EmuTOS, same image works for ACSI, SCSI and IDE testing,
+    # whereas with real TOS, different images with different drivers
+    # would be needed, potentially also for different machine types...
     hdimage   = "hd.img"
-    ideimage  = "hd.img"	 # for now use the same image as for ACSI
     hdprg     = "C:\MINIMAL.PRG"
     results   = None
 
@@ -464,34 +510,96 @@ class Tester:
     def alarm_handler(self, signum, dummy):
         "output error if (timer) signal came before passing current test stage"
         if signum == signal.SIGALRM:
-            print("ERROR: timeout triggered -> test FAILED")
+            raise OSError("ERROR: timeout")
         else:
             print("ERROR: unknown signal %d received" % signum)
             raise AssertionError
 
     def create_config(self):
-        "create Hatari configuration file for testing"
-        # write specific configuration to:
-        # - avoid user's own config
+        "create Hatari default configuration file for testing"
+        # override user's own config with default config to:
         # - get rid of the dialogs
-        # - don't warp mouse on resolution changes
+        # - disable NatFeats
+        # - disable fast boot options
+        # - run as fast as possible (fast forward)
+        # - use best CPU emulation options & 24-bit addressing
+        # - disable ST blitter & (for now) MMU
+        # - use dummy DSP & HW FPU types for speed
+        #   (OS doesn't use them, it just checks for their presence)
+        # - enable TOS patching and disable cartridge
         # - limit Videl zooming to same sizes as ST screen zooming
-        # - get rid of statusbar and borders in TOS screenshots
+        # - don't warp mouse on resolution changes
+        # - get rid of borders in TOS screenshots
         #   to make them smaller & more consistent
         # - disable GEMDOS emu by default
+        # - disable GEMDOS HD write protection and host time use
+        # - disable fast FDC & floppy write protection
         # - use empty floppy disk image to avoid TOS error when no disks
+        # - enable both floppy drives, as double sided
         # - set printer output file
         # - disable serial in and set serial output file
         # - disable MIDI in, use MIDI out as fifo file to signify test completion
         dummy = open(self.dummycfg, "w")
-        dummy.write("[Log]\nnAlertDlgLogLevel = 0\nbConfirmQuit = FALSE\n\n")
-        dummy.write("[Screen]\nnMaxWidth = 832\nnMaxHeight = 576\nbCrop = TRUE\nbAllowOverscan = FALSE\nbMouseWarp = FALSE\n\n")
-        dummy.write("[HardDisk]\nbUseHardDiskDirectory = FALSE\n\n")
-        dummy.write("[Floppy]\nszDiskAFileName = blank-a.st.gz\n\n")
-        dummy.write("[Printer]\nbEnablePrinting = TRUE\nszPrintToFileName = %s\n\n" % self.printout)
-        dummy.write("[RS232]\nbEnableRS232 = TRUE\nszInFileName = \nszOutFileName = %s\n" % self.serialout)
-        dummy.write("bEnableSccB = TRUE\nsSccBOutFileName = %s\n\n" % self.serialout)
-        dummy.write("[Midi]\nbEnableMidi = TRUE\nsMidiInFileName = \nsMidiOutFileName = %s\n\n" % self.fifofile)
+        dummy.write("""
+[Log]
+nAlertDlgLogLevel = 0
+bConfirmQuit = FALSE
+bNatFeats = FALSE
+
+[System]
+bFastBoot = FALSE
+bPatchTimerD = FALSE
+bFastForward = TRUE
+bCompatibleCpu = TRUE
+bCycleExactCpu = TRUE
+bAddressSpace24 = TRUE
+bBlitter = FALSE
+bMMU = FALSE
+nDSPType = 1
+n_FPUType = 0
+
+[ROM]
+bPatchTos = TRUE
+szCartridgeImageFileName =
+
+[Screen]
+nMaxWidth = 832
+nMaxHeight = 576
+bAllowOverscan = FALSE
+bMouseWarp = FALSE
+bCrop = TRUE
+
+[HardDisk]
+nGemdosDrive = 0
+bUseHardDiskDirectory = FALSE
+bGemdosHostTime = FALSE
+nWriteProtection = 0
+
+[Floppy]
+FastFloppy = FALSE
+nWriteProtection = 0
+EnableDriveA = TRUE
+DriveA_NumberOfHeads = 2
+EnableDriveB = TRUE
+DriveB_NumberOfHeads = 2
+szDiskAFileName = blank-a.st.gz
+
+[Printer]
+bEnablePrinting = TRUE
+szPrintToFileName = %s
+
+[RS232]
+bEnableRS232 = TRUE
+bEnableSccB = TRUE
+szInFileName =
+szOutFileName = %s
+sSccBOutFileName = %s
+
+[Midi]
+bEnableMidi = TRUE
+sMidiInFileName =
+sMidiOutFileName = %s
+""" % (self.printout, self.serialout, self.serialout, self.fifofile))
         dummy.close()
 
     def cleanup_all_files(self):
@@ -524,36 +632,26 @@ class Tester:
             if os.path.exists(path):
                 os.remove(path)
 
-    def verify_output(self, identity, tos, memory):
+    def verify_output(self, identity):
         "do verification on all test output"
-        # both tos version and amount of memory affect what
-        # GEMDOS operations work properly...
         ok = True
         # check file truncate
-        error = verify_file_empty(self.textoutput)
-        if error:
-            print("ERROR: file wasn't truncated:\n\t%s" % error)
-            os.rename(self.textoutput, "%s.%s" % (self.textoutput, identity))
-            ok = False
+        if "gemdos" in identity:
+            # file contents can be checked directly only with GEMDOS HD
+            error = verify_file_empty(self.textoutput, identity)
+            if error:
+                print("ERROR: file wasn't truncated:\n\t%s" % error)
+                ok = False
         # check serial output
-        error = verify_file_match(self.textinput, self.serialout)
+        error = verify_file_match(self.textinput, self.serialout, identity)
         if error:
             print("ERROR: serial output doesn't match input:\n\t%s" % error)
-            os.rename(self.serialout, "%s.%s" % (self.serialout, identity))
             ok = False
         # check printer output
-        error = verify_file_match(self.textinput, self.printout)
+        error = verify_file_match(self.textinput, self.printout, identity)
         if error:
-            if tos.etos or tos.version > 0x206 or (tos.version == 0x100 and memory > 1):
-                print("ERROR: printer output doesn't match input (EmuTOS, TOS v1.00 or >v2.06)\n\t%s" % error)
-                os.rename(self.printout, "%s.%s" % (self.printout, identity))
-                ok = False
-            else:
-                if os.path.exists(self.printout):
-                    error = verify_file_empty(self.printout)
-                    if error:
-                        print("WARNING: unexpected printer output (TOS v1.02 - TOS v2.06):\n\t%s" % error)
-                        os.rename(self.printout, "%s.%s" % (self.printout, identity))
+            print("ERROR: unexpected printer output:\n\t%s" % error)
+            ok = False
         self.cleanup_test_files()
         return ok
 
@@ -577,10 +675,10 @@ class Tester:
         return (False, False)
 
 
-    def open_fifo(self, timeout):
+    def open_fifo(self):
         "open FIFO for test program output"
         try:
-            signal.alarm(timeout)
+            signal.alarm(FIFO_WAIT)
             # open returns after Hatari has opened the other
             # end of fifo, or when SIGALARM interrupts it
             fifo = open(self.fifofile, "r")
@@ -591,11 +689,11 @@ class Tester:
             print("ERROR: FIFO open IOError!")
             return None
 
-    def test(self, identity, testargs, tos, memory, extrawait):
+    def test(self, identity, testargs, memwait, testwait):
         "run single boot test with given args and waits"
         # Hatari command line options, don't exit if Hatari exits
         instance = hconsole.Main(self.defaults + testargs, False)
-        fifo = self.open_fifo(tos.fullwait)
+        fifo = self.open_fifo()
         if not fifo:
             print("ERROR: failed to get FIFO to Hatari!")
             self.get_screenshot(instance, identity)
@@ -604,15 +702,16 @@ class Tester:
         else:
             init_ok = True
 
-        if tos.memwait:
+        if memwait:
+            print("Final start/memcheck wait: %ds" % memwait)
             # pass memory test
-            time.sleep(tos.memwait)
+            time.sleep(memwait)
             instance.run("keypress %s" % hconsole.Scancode.Space)
 
         # wait until test program has been run and output something to fifo
-        prog_ok, tests_ok = self.wait_fifo(fifo, tos.fullwait + extrawait)
+        prog_ok, tests_ok = self.wait_fifo(fifo, testwait)
         if tests_ok:
-            output_ok = self.verify_output(identity, tos, memory)
+            output_ok = self.verify_output(identity)
         else:
             output_ok = False
 
@@ -625,7 +724,7 @@ class Tester:
         return (init_ok, prog_ok, tests_ok, output_ok)
 
 
-    def prepare_test(self, config, tos, machine, monitor, disk, memory, ttram, extra):
+    def prepare_test(self, config, tos, machine, monitor, disk, memory, ttram, bool_opt):
         "compose test ID and Hatari command line args, then call .test()"
         identity = "%s-%s-%s-%s-%dM-%dM" % (tos.name, machine, monitor, disk, memory, ttram)
         testargs = ["--tos", tos.path, "--machine", machine, "--memsize", str(memory)]
@@ -635,10 +734,6 @@ class Tester:
             else:
                 testargs += ["--addr24", "on"]
 
-        if extra:
-            identity += "-%s%s" % (extra[0].replace("-", ""), extra[1])
-            testargs += extra
-
         if monitor.startswith("vdi"):
             planes = monitor[-1]
             testargs += ["--vdi-planes", planes]
@@ -651,12 +746,32 @@ class Tester:
         else:
             testargs += ["--monitor", monitor]
 
+        memwait = tos.memwait
+        testwait = tos.fullwait
+        mmu = False
+        if bool_opt:
+            if bool_opt[0] == '--mmu' and bool_opt[1] == 'on':
+                mmu = True
+            identity += "-%s%s" % (bool_opt[0].replace('-', ''), bool_opt[1])
+            testargs += bool_opt
+        if config.opts:
+            if "--mmu" in config.opts:
+                mmu = True
+            # pass-through Hatari options
+            testargs += config.opts
+        if mmu and machine in ("tt", "falcon"):
+            # MMU doubles memory wait
+            memwait *= 2
+
         if config.fast:
             testargs += ["--fast-forward", "yes", "--fast-boot", "yes",
                          "--fastfdc", "yes", "--timer-d", "yes"]
+        elif machine == "falcon" and disk != "ide":
+            # Falcon IDE interface scanning when there's no IDE takes long
+            memwait += 8
 
-        extrawait = 0
         if disk == "gemdos":
+            exit_if_missing([self.testprg, self.textinput])
             # use Hatari autostart, must be last thing added to testargs!
             testargs += [self.testprg]
         # HD supporting TOSes support also INF file autostart, so
@@ -670,21 +785,26 @@ class Tester:
         #    with --auto
         elif disk == "floppy":
             if tos.supports_gemdos_hd():
+                exit_if_missing([self.bootdesk])
                 testargs += ["--disk-a", self.bootdesk, "--auto", self.floppyprg]
             else:
+                exit_if_missing([self.bootauto])
                 testargs += ["--disk-a", self.bootauto]
             # floppies are slower
-            extrawait = 3
+            testwait += 3
         elif disk == "acsi":
+            exit_if_missing([self.hdimage])
             testargs += ["--acsi", "0=%s" % self.hdimage, "--auto", self.hdprg]
         elif disk == "scsi":
+            exit_if_missing([self.hdimage])
             testargs += ["--scsi", "0=%s" % self.hdimage, "--auto", self.hdprg]
         elif disk == "ide":
-            testargs += ["--ide-master", self.ideimage, "--auto", self.hdprg]
+            exit_if_missing([self.hdimage])
+            testargs += ["--ide-master", self.hdimage, "--auto", self.hdprg]
         else:
             raise AssertionError("unknown disk type '%s'" % disk)
 
-        results = self.test(identity, testargs, tos, memory, extrawait)
+        results = self.test(identity, testargs, memwait, testwait)
         self.results[tos.name].append((identity, results))
 
     def run(self, config):
@@ -705,20 +825,21 @@ class Tester:
                     for memory in config.memsizes:
                         if not config.valid_memsize(machine, memory):
                             continue
-                        for ttram in config.ttrams:
-                            if not config.valid_ttram(machine, tos, ttram, self.winuae):
+                        for disk in config.disks:
+                            if not config.valid_disktype(machine, tos, disk):
                                 continue
-                            for disk in config.disks:
-                                if not config.valid_disktype(machine, tos, disk):
+                            for ttram in config.ttrams:
+                                if not config.valid_ttram(machine, tos, ttram, disk, self.winuae):
                                     continue
-                                if config.bools:
-                                    for opt in config.bools:
-                                        if not config.valid_bool(machine, opt):
-                                            continue
-                                        for val in ('on', 'off'):
-                                            self.prepare_test(config, tos, machine, monitor, disk, memory, ttram, [opt, val])
-                                            count += 1
-                                else:
+                                no_bools = True
+                                for opt in config.bools:
+                                    if not config.valid_bool(machine, opt):
+                                        continue
+                                    no_bools = False
+                                    for val in ('on', 'off'):
+                                        self.prepare_test(config, tos, machine, monitor, disk, memory, ttram, [opt, val])
+                                        count += 1
+                                if no_bools:
                                     self.prepare_test(config, tos, machine, monitor, disk, memory, ttram, None)
                                     count += 1
             if not count:
@@ -765,7 +886,7 @@ class Tester:
         report.write("\n")
 
         # print report out too
-        print("--- %s ---" % self.report)
+        print("\n--- %s ---" % self.report)
         report = open(self.report, "r")
         for line in report.readlines():
             print(line.strip())
@@ -776,6 +897,8 @@ def main():
     "tester main function"
     info = "Hatari TOS bootup tester"
     print("\n%s\n%s\n" % (info, "-"*len(info)))
+    # avoid global config file
+    os.environ["HATARI_TEST"] = "boot"
     config = Config(sys.argv)
     tester = Tester()
     tester.run(config)
